import { LRLanguage, LanguageSupport } from "@codemirror/language";
import { ViewPlugin, ViewUpdate, DecorationSet } from "@codemirror/view";
import { Diagnostic } from "@codemirror/lint";
declare const NuformulaLanguage: LRLanguage;
declare function nuformula(): LanguageSupport;
// 表單元件物件格式
type Item = {
    sn: string;
    type: string;
    column: boolean;
    parent: string;
    label: string;
    options?: {
        value: string;
        label: string;
    }[];
};
// 可用表單項目類型字串
declare enum widgetType {
    input = "input",
    // ----------------------- 單行輸入
    textarea = "textarea",
    // ----------------- 多行輸入
    number = "number",
    // --------------------- 數字
    radio = "radio",
    // ----------------------- 單選
    checkbox = "checkbox",
    // ----------------- 複選
    select = "select",
    // --------------------- 下拉單選
    selectMultiple = "select_multiple"
}
declare enum opName {
    /* 數學運算子 */
    add = "add",
    // ---------------------- 加法 (+)
    subtract = "subtract",
    // -----------  減法 (-)
    time = "time",
    // -------------------- 乘法 (*)
    divide = "divide",
    // ---------------- 除法 (/)
    eq = "eq",
    // ------------------------ 等於 (=)
    gt = "gt",
    // ------------------------ 大於 (>)
    lt = "lt",
    // ------------------------ 小於 (<)
    gte = "gte",
    // ---------------------- 大於或等於 (>=)
    lte = "lte",
    // ---------------------- 小於或等於 (<=)
    neq = "neq",
    /* 函式運算子 */
    equal = "equal"
}
// 可用函式
declare enum funcName {
    SUM = "sum",
    // --------------------------- 加總
    AVERAGE = "average",
    // ------------------- 平均
    IF = "if",
    // ----------------------------- 條件
    CONCAT = "concat"
}
/**
 * 輸出 表單元件 decoration 插件
 *
 * @param {object} formItems - 所有表單元件
 * @returns {ViewPlugin}
 */
declare function nuformulaItemWidget(formItems: {
    [key: string]: Item;
}): ViewPlugin<{
    decorations: DecorationSet;
    update(update: ViewUpdate): void;
}>;
declare class NuLinter {
    text: string; // -------------------- 公式字串
    formItemsKeys: string[]; // --------- 表單元件鍵值
    private length; // ---------- 公式字串長度
    private prevChar; // -------- 前一字符
    private char; // ------------ 當前字符
    private nextChar; // -------- 後一字符
    private pStack; // -------- 括號配對佇列
    private qStack; // -------- 引號配對佇列
    private iStack; // -------- 表單項目配對佇列
    private fStack; // ------- 函式參數佇列
    private itemSn; // ---------- 表單元件項目
    private funcName; // -------- 函式名稱
    constructor();
    init(formItemsKeys: string[]): void;
    /**
     * 重置 linter
     *
     * @param {string} text - 公式字串Ｆ
     */
    reset(text?: string): void;
    /**
     * 驗證公式
     *
     * @param {string} text - 公式字串
     * @returns {object} - 錯誤訊息
     */
    verify(text: string): {
        from: number;
        to: number;
        severity: string;
        message: string;
    } | null;
}
declare const nuLint: NuLinter;
declare const nuformulaLinter: (formItemsKeys: string[], callback: (error: Diagnostic) => void) => import("@codemirror/state").Extension;
// 語言標籤樣式
declare function nuformulaHighlightStyle(): import("@codemirror/state").Extension;
/**
 * 輸出 自動選字 掛件
 *
 * @param {Object} options - 可選選項
 * @returns {Extension}
 */
declare function nuformulaAutocomplete(formItems: {
    [key: string]: Item;
}): import("@codemirror/state").Extension;
// 參數物件格式
type Argument = {
    op: string; // ----------- 運算子或運算元
    type: string; // --------- 參數類型
    value: any; // ----------- 參數攜帶值
    argCount?: number; // ---- 函式參數數量
};
declare class FormulaEvaluation {
    // 表單元件
    formItems: {
        [key: string]: Item;
    };
    constructor();
    /**
     * 初始化公式計算工具
     *
     * @param {object} formItems - 所有可用表單元件
     */
    init(formItems: {
        [key: string]: Item;
    }): void;
    /**
     * 取得公式計算方法
     *
     * @param {Array} postfixStack - postfix 公式佇列
     * @param {string} assignTarget - 公式賦值的元件序號
     * @returns {Function}
     */
    getEvaluation(postfixStack: Argument[], assignTarget?: string): (scopedData: {
        [key: string]: any;
    }, row?: string) => string | number;
    getFormattedValue(type: string, arg: string, data: any): string | number | string[];
    /**
     * 取得 單選 radio / 下拉單選 select 標籤字串
     *
     * @param {string} itemSn - 表單項目
     * @param {string} value - 填寫值
     * @returns {string}
     */
    findStringOptionLabel(itemSn: string, value: string | undefined): string;
    /**
     * 取得 單選 radio / 下拉單選 select 標籤字串
     *
     * @param {string} itemSn - 表單項目
     * @param {Array} value - 填寫值
     * @param {string} type - 選項鍵值
     * @returns {string}
     */
    findArrayOptionLabel(itemSn: string, value: string[] | undefined): string[];
}
declare const nuformulaEvaluation: FormulaEvaluation;
/**
 * ==================================
 * 參考 Microsoft Excel 規格
 *
 * @see https://support.microsoft.com/zh-tw/office/78be92ad-563c-4d62-b081-ae6da5c2ca69
 * @see https://support.microsoft.com/zh-tw/office/5f91f4e9-7b42-46d2-9bd1-63f26a86c0eb
 * ==================================
 */
declare const NuEvaluationLib: {
    add: (args: any[]) => number;
    subtract: (args: any[]) => number;
    time: (args: any[]) => number;
    divide: (args: any[]) => number;
    eq: (args: any[]) => boolean;
    gt: (args: any[]) => boolean;
    lt: (args: any[]) => boolean;
    gte: (args: any[]) => boolean;
    lte: (args: any[]) => boolean;
    neq: (args: any[]) => boolean;
    equal: (args: any[]) => any;
    sum: (args: any[]) => number;
    if: (args: any[]) => any;
    concat: (args: any[]) => string;
    average: (args: any[]) => number;
};
declare function formatFormItems(formItems: {
    [key: string]: any;
}, config?: {
    summaryLabel?: string;
}): {
    [key: string]: Item;
};
export { NuformulaLanguage, nuformula, nuformulaItemWidget, nuLint, nuformulaLinter, nuformulaHighlightStyle, nuformulaAutocomplete, nuformulaEvaluation, NuEvaluationLib, Item, widgetType, opName, funcName, formatFormItems };
